package cn.com.qmth.examcloud.task.service.job;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Component;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.common.dto.core.enums.CourseLevel;
import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.task.base.AbstractTask;
import cn.com.qmth.examcloud.task.base.ScheduleJob;
import cn.com.qmth.examcloud.task.base.TaskTracker;
import cn.com.qmth.examcloud.task.dao.ExamStudentImportRepo;
import cn.com.qmth.examcloud.task.dao.ExamStudentTempRepo;
import cn.com.qmth.examcloud.task.dao.entity.ExamStudentImportEntity;
import cn.com.qmth.examcloud.task.dao.entity.ExamStudentTempEntity;
import cn.com.qmth.examcloud.task.dao.enums.ExamStudentImportStatus;

/**
 * 考生导入-文件解析
 *
 * @author WANGWEI
 * @date 2018年7月31日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Component("examStudentImportDataProcessingTask")
public class ExamStudentImportDataProcessingTask extends AbstractTask {

	@Value("${$dir}")
	private String dir;

	private static final String EXAM_STUDENT_IMPORT_FILES = "exam_student_import_files";

	@Autowired
	TaskTracker TaskTracker;

	@Autowired
	ExamStudentImportRepo examStudentImportRepo;

	@Autowired
	ExamStudentTempRepo examStudentTempRepo;

	@Autowired
	CourseCloudService courseCloudService;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamCloudService examCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@Override
	public void run(ScheduleJob scheduleJob) throws Exception {

		ExamStudentImportEntity importEntity = examStudentImportRepo
				.findFirstByStatusOrderByCreationTime(
						ExamStudentImportStatus.FILE_PARSING_COMPLETE);
		if (null == importEntity) {
			return;
		}

		importEntity.setStatus(ExamStudentImportStatus.DATA_PROCESSING);
		examStudentImportRepo.saveAndFlush(importEntity);

		process(importEntity);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param importEntity
	 */
	private void process(ExamStudentImportEntity importEntity) {
		List<Map<String, Object>> failRecords = Collections
				.synchronizedList(new ArrayList<Map<String, Object>>());

		foreach(importEntity, failRecords);

		StringBuilder sb = new StringBuilder();
		for (Map<String, Object> cur : failRecords) {
			sb.append("line:").append(cur.get("lineNum")).append(cur.get("msg")).append("\n");
		}
		sb.append("==============================================\n");
		String resultFilePath = PathUtil.getCanonicalPath(
				dir + "/" + EXAM_STUDENT_IMPORT_FILES + "/" + importEntity.getResultFilePath());
		File resultFile = new File(resultFilePath);

		try {
			FileUtils.writeStringToFile(resultFile, sb.toString(), "UTF-8", true);
		} catch (IOException e) {
			debugLog.error("导入报告写入失败", e);
			importEntity.setStatus(ExamStudentImportStatus.ERROR);
			importEntity.setErrorDesc("导入报告写入失败");
			examStudentImportRepo.saveAndFlush(importEntity);
			return;
		}

		importEntity.setStatus(ExamStudentImportStatus.DATA_PROCESSING_COMPLETE);
		examStudentImportRepo.saveAndFlush(importEntity);

	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param importEntity
	 * @param failRecords
	 */
	private void foreach(ExamStudentImportEntity importEntity,
			List<Map<String, Object>> failRecords) {
		Long batchId = importEntity.getBatchId();

		while (true) {
			Pageable pageable = new PageRequest(0, 100, Sort.Direction.DESC, "updateTime");
			List<ExamStudentTempEntity> list = examStudentTempRepo
					.findByBatchIdAndStatusCodeIsNull(batchId, pageable);
			if (CollectionUtils.isEmpty(list)) {
				break;
			}

			for (ExamStudentTempEntity cur : list) {
				saveExamStudent(cur, failRecords);
			}
		}

	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param entity
	 * @param failRecords
	 */
	private void saveExamStudent(ExamStudentTempEntity entity,
			List<Map<String, Object>> failRecords) {
		Long rootOrgId = entity.getRootOrgId();
		String courseCode = entity.getCourseCode();
		String orgCode = entity.getOrgCode();

		try {
			GetCourseReq req = new GetCourseReq();
			req.setRootOrgId(rootOrgId);
			req.setCode(courseCode);
			GetCourseResp resp = courseCloudService.getCourse(req);
			CourseBean courseBean = resp.getCourseBean();
			entity.setCourseId(courseBean.getId());
		} catch (StatusException e) {
			debugLog.error("查询课程异常", e);
			Map<String, Object> map = Maps.newHashMap();
			map.put("lineNum", entity.getLineNum());
			map.put("msg", e.getDesc());
			failRecords.add(map);
			entity.setStatusCode(e.getCode());
			entity.setStatusDesc(e.getDesc());
			examStudentTempRepo.saveAndFlush(entity);
			return;
		}

		try {
			GetOrgReq req = new GetOrgReq();
			req.setOrgCode(orgCode);
			req.setRootOrgId(rootOrgId);
			GetOrgResp resp = orgCloudService.getOrg(req);
			OrgBean org = resp.getOrg();
			entity.setOrgId(org.getId());
		} catch (StatusException e) {
			debugLog.error("查询机构异常", e);
			Map<String, Object> map = Maps.newHashMap();
			map.put("lineNum", entity.getLineNum());
			map.put("msg", e.getDesc());
			failRecords.add(map);
			entity.setStatusCode(e.getCode());
			entity.setStatusDesc(e.getDesc());
			examStudentTempRepo.saveAndFlush(entity);
			return;
		}

		try {
			InsertOrUpdateStudentReq saveStudentReq = new InsertOrUpdateStudentReq();
			saveStudentReq.setIdentityNumber(entity.getIdentityNumber());
			saveStudentReq.setName(entity.getName());
			saveStudentReq.setPhoneNumber(entity.getPhone());
			saveStudentReq.setOrgCode(entity.getOrgCode());
			saveStudentReq.setOrgId(entity.getOrgId());
			saveStudentReq.setRootOrgId(rootOrgId);
			saveStudentReq.setStudentCode(entity.getStudentCode());

			InsertOrUpdateStudentResp insertOrUpdateStudentResp = studentCloudService
					.insertOrUpdateStudent(saveStudentReq);

			Long studentId = insertOrUpdateStudentResp.getStudentId();

			SaveExamStudentReq sReq = new SaveExamStudentReq();
			sReq.setCourseLevel(entity.getCourseLevel());

			CourseLevel courseLevel = CourseLevel.getCourseLevel(entity.getCourseLevel());
			sReq.setCourseLevel(courseLevel.name());
			sReq.setCourseCode(entity.getCourseCode());
			sReq.setCourseName(entity.getCourseName());
			sReq.setExamId(entity.getExamId());
			sReq.setStudentId(studentId);
			sReq.setIdentityNumber(entity.getIdentityNumber());
			sReq.setPaperType(entity.getPaperType());
			sReq.setRootOrgId(rootOrgId);
			sReq.setStudentCode(entity.getStudentCode());
			sReq.setStudentName(entity.getName());
			examStudentCloudService.saveExamStudent(sReq);

			entity.setStatusCode("200");
			entity.setStatusDesc("成功");
			examStudentTempRepo.saveAndFlush(entity);

		} catch (StatusException e) {
			debugLog.error("考生入库异常", e);
			Map<String, Object> map = Maps.newHashMap();
			map.put("lineNum", entity.getLineNum());
			map.put("msg", "  " + e.getDesc());
			failRecords.add(map);
			entity.setStatusCode(e.getCode());
			entity.setStatusDesc(e.getDesc());
			examStudentTempRepo.saveAndFlush(entity);
			return;
		}

	}

	@Override
	public TaskTracker getTaskTracker() {
		return TaskTracker;
	}
}
